%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function estimates the group memberships and slope parameters 
% for given gamma in the panel structural threshold regression model 
% with known threshold value
%
% Input: 
% idxdata = data
% G = the number of groups
% gamma = given threshold
% 
% Output:
% [theta1_best theta2_best] = estimated slope coefficients
% gamma_best = estimated threshold
% gi_best = estimated group membership
% resQ_best = sum of squared residuals
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [theta1_best, theta2_best, gi_best, resQ_best, resid_best, se1, se2] = est_pstr_known_threshold(idxdata,G,gamma,h)

N               = idxdata.N(end); 
T               = size(idxdata,1)/N;
k               = size(idxdata.X,2);
Y               = idxdata.y;
X               = idxdata.X;
q               = idxdata.q;

%% estimation
Nsim            = 100;                                                      % number of simulations
resQ_best       = 10^7;                                                     % sum of squared residuals for the best initial values
max_iteration   = 20;

for jsim = 1:Nsim                                                           % use different initial values 
    gi_init     = zeros(N,G);                                                   
    for i = 1:N                                                             % initialize a random group pattern
        perm    = randperm(G);    
        for g = 1:G    
            gi_init(i,g) = (perm(1)==g);        
        end    
    end
    gi          = gi_init;    
    par_init    = zeros(N*T,k);
        
    deltapar    = 1;                                                        % iteration indicator   
    num_iter    = 0;                                                        % number of iterations
    while deltapar > 0 && num_iter < max_iteration                          % iterated optimization
        if ~all(sum(gi))
            for i = 1:N                                                     % if any group has no element, start a new random initialization
            perm                    = randperm(G);    
                for g = 1:G    
                    gi_init(i,g)    = (perm(1)==g);        
                end    
            end
            gi                      = gi_init;           
        end
        %% step 1: update          
        for g = 1:G 
            NN                      = 1:N;
            gi                      = logical(gi);
            this_group              = gi(:,g);    
            g_index                 = NN(this_group);    
            g_data                  = idxdata(ismember(idxdata.N,g_index),:);   % group-specific data	
            Ng                      = sum(gi(:,g));                             % number of individuals within this group
            gX                      = g_data.X;
            gy                      = g_data.y;
            gq                      = g_data.q;            
                        
            i1                      = (gq<=gamma);                          % indicator of the first subsample    
            i2                      = ~(i1);                                % indicator of the second subsample                
            x1                      = gX(i1,:);    
            y1                      = gy(i1,:);    
            x2                      = gX(i2,:);    
            y2                      = gy(i2,:);
            hattheta1(g,:)          = ((x1'*x1)\(x1'*y1))';
            hattheta2(g,:)          = ((x2'*x2)\(x2'*y2))';            
        end
        
        %% step 2: group assignment
        for g = 1:G        
            hattheta1ful            = repmat(hattheta1(g,:),N*T,1);
            hattheta2ful            = repmat(hattheta2(g,:),N*T,1);
            gammaful                = repmat(gamma,N*T,1);
            hati1                   = repmat((q <= gammaful),1,k);
            hati2                   = repmat((q > gammaful),1,k);
            U                       = (Y-sum(X.*hati1.*hattheta1ful,2) - sum(X.*hati2.*hattheta2ful,2)).^2;              % squared residual using beta of the g-th group  
            RU                      = reshape(U,T,N)';                   
            gi_aux(:,g)             = sum(RU')';                            % time summation of squared residual for the g-th group
        end    
        [gi_class, gindt]           = min(gi_aux,[],2);                     % for each individual, find the group with least sum of squared residual
        for g = 1:G
            gi(:,g)                 = (gi_aux(:,g) == gi_class);            % assign each individual to the optimal group       
        end

        %% check convergence
        par_new                     = zeros(N*T,k);
        gamma_new                   = zeros(N*T,1);
        theta1_new                  = zeros(N*T,k);
        theta2_new                  = zeros(N*T,k);
        for i = 1:N     
            gamma_new((i-1)*T+1:i*T,1)  = repmat(gamma,T,1);
            theta1_new((i-1)*T+1:i*T,:) = repmat(hattheta1(gindt(i),:),T,1);
            theta2_new((i-1)*T+1:i*T,:) = repmat(hattheta2(gindt(i),:),T,1);            
        end
        i1_new                      = repmat((q <= gamma_new),1,k);
        i2_new                      = repmat((q > gamma_new),1,k);
        par_new                     = i1_new.*theta1_new + i2_new.*theta2_new;  % updated parameter for each individual         
        deltapar                    = norm(par_new-par_init);                   % update iteration indicator: check norm between updated and initial parameter
        par_init                    = par_new;    
        num_iter                    = num_iter + 1;
    end
    resid                           = (Y-sum(X.*par_new,2));
    resQ                            = (Y-sum(X.*par_new,2))'*(Y-sum(X.*par_new,2));                
    if resQ < resQ_best
        resQ_best                   = resQ;
        resid_best                  = resid;
        gi_best                     = gi;
        theta1_best                 = hattheta1;
        theta2_best                 = hattheta2;
    end        
end

%% post-group estimation
for g = 1:G 
    NN                      = 1:N;    
    gi                      = logical(gi_best);    
    this_group              = gi(:,g);        
    g_index                 = NN(this_group);        
    g_data                  = idxdata(ismember(idxdata.N,g_index),:);       % group-specific data	    
    Ng                      = sum(gi(:,g));                                 % number of individuals within this group    
    gX                      = g_data.X;    
    gy                      = g_data.y;    
    gq                      = g_data.q;            
    i1                      = (gq<=gamma);                                  % indicator of the first subsample        
    i2                      = ~(i1);                                        % indicator of the second subsample                    
    x1                      = gX(i1,:);        
    y1                      = gy(i1,:);        
    x2                      = gX(i2,:);        
    y2                      = gy(i2,:);    
    mi1                     = inv(x1'*x1);
    mi2                     = inv(x2'*x2);
    hattheta1(g,:)          = ((x1'*x1)\(x1'*y1))';    
    hattheta2(g,:)          = ((x2'*x2)\(x2'*y2))';     
    e1                      = y1-x1*hattheta1(g,:)';
    e2                      = y2-x2*hattheta2(g,:)';
    if h == 0
        se1(g,:)            = sqrt(diag(mi1)*sigful);
        se2(g,:)            = sqrt(diag(mi2)*sigful);
    elseif h == 1
        se1(g,:)            = sqrt(diag(mi1*(x1.*(e1*(ones(1,length(x1(1,:))))))'*(x1.*(e1*(ones(1,length(x1(1,:))))))*mi1));
        se2(g,:)            = sqrt(diag(mi2*(x2.*(e2*(ones(1,length(x2(1,:))))))'*(x2.*(e2*(ones(1,length(x2(1,:))))))*mi2));
    elseif h == 2   
        [~,se1(g,:),~]      = hac(x1,y1,'intercept',false,'display','off','bandwidth',0.9);
        [~,se2(g,:),~]      = hac(x2,y2,'intercept',false,'display','off','bandwidth',0.9);    
    end
end
