%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function computes the p-values of two types of tests for the
% existence of the threshold effect
% Input: 
% y = dependent variable
% x = independent variable
% q = threshold variable
% h = heteroskedasticity indicator
% N = number of individuals
% alpha = confidence level
%
% Output:
% pmax = p-value of supsup test
% psum = p-value of supsum test
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [pmax,psum]    = test_existence_threshold(y,x,q,N,G,h)


NT              = length(y);
T               = NT/N;
k               = size(x,2);

%% within transformation
Ry              = reshape(y,T,N);
ydm             = Ry - repmat(mean(Ry),T,1);
for j = 1:k
    Rx          = reshape(x(:,j),T,N);
    Rxdm        = Rx - repmat(mean(Rx),T,1);
    xdm(:,j)    = reshape(Rxdm,N*T,1);
end
Y               = reshape(ydm,N*T,1);
X               = reshape(xdm,N*T,k);

time            = repmat(1:T,1,N)';
code            = kron((1:N)',ones(T,1));
idxdata         = dataset(code, time, Y, X, q);
idxdata.Properties.VarNames = {'N'  'T'  'y'  'X' 'q'};

%% split-sample estimation for different threshold
% qs              = sort(unique(q));
qs              = quantile(q,[0.2:0.01:0.8]);
qn              = length(qs);
censor          = 0.3;                                                      % trim extreme threshold
qn_start        = round(censor*qn);
qn_end          = round((1-censor)*qn);
B               = 600;                                                      % number of simulation times for critical values
if h == 1
    vv          = normrnd(0,1,N*T,B*k);
elseif h == 2
    Sigmavv     = zeros(T,T);
    for t = 1:T
        for j = 1:T-t
            bandw           = 0.05;
            u               = j/bandw;
            if abs(u)<=1
                kh(t,j)     = 1-u;
            else
                kh(t,j)     = 0;
            end
            Sigmavv(t,t+j)  = kh(t,j);
        end
    end
    Sigmavv     = Sigmavv + Sigmavv' + diag(ones(T,1));
    for i = 1:N
        vv((i-1)*T+1:i*T,:)   = mvnrnd(zeros(T,1),Sigmavv,B*k)';
    end
end
W               = zeros(qn_end-qn_start+1,G);
Tn              = zeros(qn_end-qn_start+1,G,B);        
Mn              = zeros(2*k*G,2*k,qn_end-qn_start+1);
gxe             = zeros(N,2*k*G,qn_end-qn_start+1);
Vstar           = zeros(2*k*G,2*k,qn_end-qn_start+1);

ss              = 1;    
for rr = qn_start:qn_end
    gamma           = qs(rr);
    [est_theta1, est_theta2, est_gi, ~, res] = est_pstr_known_threshold(idxdata,G,gamma,h); % group estimation for each given gamma
    resid(:,rr-qn_start+1)     = res;
    for g = 1:G
        NN          = 1:N;
        gi          = logical(est_gi);
        this_group  = gi(:,g);
        g_index     = NN(this_group);
        g_data      = idxdata(ismember(idxdata.N,g_index),:);               % group-specific data
        Ng          = sum(gi(:,g));                                         % number of individuals within this group
        gX          = g_data.X;
        gy          = g_data.y;
        gq          = g_data.q;
        ggammaful   = repmat(gamma,Ng*T,1);
        gthetaful1  = repmat(est_theta1(g,:),Ng*T,1);
        gthetaful2  = repmat(est_theta2(g,:),Ng*T,1);
        gi1         = repmat((gq <= ggammaful),1,k);
        gi2         = repmat((gq > ggammaful),1,k);
        gx1         = gX.*gi1;
        gx2         = gX.*gi2;
        gres        = (gy-sum(gx1.*gthetaful1,2) - sum(gx2.*gthetaful2,2)); % residual using estimate of the g-th group
        
        gx                              = [gx1 gx2];
        Mn(2*k*(g-1)+1:2*k*g,:,ss)      = gx'*gx;
        gxe_tmp                         = gx.*repmat(gres,1,2*k);           % score function
        lg_gxe(g)                       = length(gxe_tmp);
        gxe((g-1)*lg_gxe(g)+1:g*lg_gxe(g),2*k*(g-1)+1:2*k*g,ss) = gxe_tmp;  
        if h == 1
            Vn                          = gxe((g-1)*lg_gxe(g)+1:g*lg_gxe(g),2*k*(g-1)+1:2*k*g,ss)'*gxe((g-1)*lg_gxe(g)+1:g*lg_gxe(g),2*k*(g-1)+1:2*k*g,ss);
            Vstar(2*k*(g-1)+1:2*k*g,:,ss)   = Mn(2*k*(g-1)+1:2*k*g,:,ss)\Vn/Mn(2*k*(g-1)+1:2*k*g,:,ss);
        elseif h == 2
            Vstar(2*k*(g-1)+1:2*k*g,:,ss)   = hac(gx,gy,'intercept',false,'display','off','bandwidth',2);
        end        
        R                               = [eye(k) -eye(k)];
        gtheta                          = [est_theta1(g,:)'; est_theta2(g,:)'];        
        W(ss,g)                         = (R*gtheta)'/(R*Vstar(2*k*(g-1)+1:2*k*g,:,ss)*R')*(R*gtheta);
    end
    ss = ss + 1;
end

%% simulated statistics    
for s = 1:ss-1
    for g = 1:G        
        repmatv     = kron(vv(1:lg_gxe(g),:),ones(1,2));                
        repmatgxe   = repmat(gxe((g-1)*lg_gxe(g)+1:g*lg_gxe(g),2*k*(g-1)+1:2*k*g,s),1,B);                
        repmatSn    = (sum(repmatgxe.*repmatv))';        
        for b = 1:B        
            Sn          = repmatSn((b-1)*2*k+1:b*2*k);             
            Tn(s,g,b)   = Sn'/Mn(2*k*(g-1)+1:2*k*g,:,s)*R'/(R*Vstar(2*k*(g-1)+1:2*k*g,:,s)*R')*R/Mn(2*k*(g-1)+1:2*k*g,:,s)*Sn;                    
        end        
    end    
end

Tmax                = max(max(Tn));
Waldmax             = max(max(W));
pmax                = 1-sum(Waldmax>sort(Tmax))/length(Tmax);
Tsum                = max(sum(Tn));
Waldsum             = max(sum(W));
psum                = 1-sum(Waldsum>sort(Tsum))/length(Tsum);

end